local VERSION = tonumber(("$Revision: 43176 $"):match("%d+"))

local Parrot = Parrot
local Parrot_CombatEvents = Parrot:NewModule("CombatEvents", "Parser-3.0", "AceEvent-2.0")
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-08 02:27:21 -0400 (Sun, 08 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local L = Parrot:L("Parrot_CombatEvents", {
	["Miss!"] = MISS .. "!",
	["Dodge!"] = DODGE .. "!",
	["Parry!"] = PARRY .. "!",
	["Block!"] = BLOCK .. "!",
	["Absorb!"] = ABSORB .. "!",
	["Immune!"] = IMMUNE .. "!",
	["Resist!"] = RESIST .. "!",
	["Reflect!"] = REFLECT .. "!",
	["Interrupt!"] = INTERRUPT .. "!",
	["Pet Miss!"] = PET .. " " .. MISS .. "!",
	["Pet Dodge!"] = PET .. " " .. DODGE .. "!",
	["Pet Parry!"] = PET .. " " .. PARRY .. "!",
	["Pet Block!"] = PET .. " " .. BLOCK .. "!",
	["Pet Absorb!"] = PET .. " " .. ABSORB .. "!",
	["Pet Immune!"] = PET .. " " .. IMMUNE .. "!",
	["Pet Resist!"] = PET .. " " .. RESIST .. "!",
	["Pet Reflect!"] = PET .. " " .. REFLECT .. "!",
	["Pet Interrupt!"] = PET .. " " .. INTERRUPT .. "!",
})

local Gratuity -- AceLibrary("Gratuity-2.0")
local Aura -- AceLibrary("SpecialEvents-Aura-2.0")

local new, del = Parrot.new, Parrot.del

Parrot_CombatEvents.db = Parrot:AcquireDBNamespace("CombatEvents")
Parrot:RegisterDefaults("CombatEvents", "profile", {
	['*'] = {
		['*'] = {}
	}
})

local Parrot_Display
local onEnableFuncs = {}
function Parrot_CombatEvents:OnEnable()
	Parrot_Display = Parrot:GetModule("Display")
	Gratuity = AceLibrary("Gratuity-2.0")
	Aura = AceLibrary("SpecialEvents-Aura-2.0")
	
	for _,v in ipairs(onEnableFuncs) do
		v()
	end
end

local onDisableFuncs = {}
function Parrot_CombatEvents:OnDisable()
	for _,v in ipairs(onDisableFuncs) do
		v()
	end
end

Parrot.options.args.events = {
	type = 'group',
	name = L["Events"],
	desc = L["Change event settings"],
	args = {
		Incoming = {
			type = 'group',
			name = L["Incoming"],
			desc = L["Incoming events are events which a mob or another player does to you."],
			args = {},
		},
		Outgoing = {
			type = 'group',
			name = L["Outgoing"],
			desc = L["Outgoing events are events which you do to a mob or another player."],
			args = {},
		},
		Notification = {
			type = 'group',
			name = L["Notification"],
			desc = L["Notification events are available to notify you of certain actions."],
			args = {},
		},
	}
}

local combatEvents = {}

local function getTag(passValue)
	local category, name = (";"):split(passValue, 2)
	return Parrot_CombatEvents.db.profile[category][name].tag or combatEvents[category][name].defaultTag
end
local function setTag(passValue, value)
	local category, name = (";"):split(passValue, 2)
	if combatEvents[category][name].defaultTag == value then
		value = nil
	end
	Parrot_CombatEvents.db.profile[category][name].tag = value
end

local function hexColorToTuple(color)
	local num = tonumber(color, 16)
	return math.floor(num / 256^2)/255, math.floor((num / 256)%256)/255, (num%256)/255
end

local function tupleToHexColor(r, g, b)
	return ("%02x%02x%02x"):format(r * 255, g * 255, b * 255)
end

local function getColor(passValue)
	local category, name = (";"):split(passValue, 2)
	return hexColorToTuple(Parrot_CombatEvents.db.profile[category][name].color or combatEvents[category][name].color)
end
local function setColor(passValue, r, g, b)
	local category, name = (";"):split(passValue, 2)
	local color = tupleToHexColor(r, g, b)
	local combatEvent = combatEvents[category][name]
	if combatEvent.color == color then
		color = nil
	end
	self.db.profile[category][name].color = color
end

local function getSticky(passValue)
	local category, name = (";"):split(passValue, 2)
	local sticky = Parrot_CombatEvents.db.profile[category][name].sticky
	if sticky ~= nil then
		return sticky
	else
		return combatEvents[category][name].sticky
	end
end
local function setSticky(passValue, value)
	local category, name = (";"):split(passValue, 2)
	if combatEvents[category][name].sticky == value then
		value = nil
	end
	Parrot_CombatEvents.db.profile[category][name].sticky = value
end

function Parrot_CombatEvents:RegisterCombatEvent(data)
	self = Parrot_CombatEvents -- so people can do Parrot:RegisterCombatEvent
	AceLibrary.argCheck(self, data, 2, "table")
	local category = data.category
	if type(category) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. category must be a %q, got %q."):format("string", type(category)), 2)
	end
	local name = data.name
	if type(name) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. name must be a %q, got %q."):format("string", type(name)), 2)
	end
	local localName = data.localName
	if type(localName) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. localName must be a %q, got %q."):format("string", type(localName)), 2)
	end
	local defaultTag = data.defaultTag
	if type(defaultTag) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. defaultTag must be a %q, got %q."):format("string", type(defaultTag)), 2)
	end
	local parserEvent = data.parserEvent
	if parserEvent and type(parserEvent) ~= "table" then
		error(("Bad argument #2 to `RegisterCombatEvent'. parserEvent must be a %q or nil, got %q."):format("table", type(parserEvent)), 2)
	end
	local blizzardEvent = data.blizzardEvent
	if blizzardEvent and type(blizzardEvent) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. blizzardEvent must be a %q or nil, got %q."):format("string", type(blizzardEvent)), 2)
	end
	if parserEvent and blizzardEvent then
		error("Bad argument #2 to `RegisterCombatEvent'. blizzardEvent and parserEvent cannot coexist.", 2)
	end
	local tagTranslations = data.tagTranslations
	if tagTranslations and type(tagTranslations) ~= "table" then
		error(("Bad argument #2 to `RegisterCombatEvent'. tagTranslations must be a %q or nil, got %q."):format("table", type(tagTranslations)), 2)
	end
	local color = data.color
	if type(color) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. color must be a %q, got %q."):format("string", type(color)), 2)
	end
	if not combatEvents[category] then
		combatEvents[category] = new()
	end
	combatEvents[category][name] = data
	if blizzardEvent then
		self:RegisterEvent(blizzardEvent, function(...)
			local info = new(...)
			self:TriggerCombatEvent(category, name, info)
			info = del(info)
		end)
	elseif parserEvent then
		self:RegisterParserEvent(parserEvent, self.TriggerCombatEvent, self, category, name)
	end
	
	Parrot.options.args.events.args[category].args[name] = {
		type = 'group',
		name = localName,
		desc = localName,
		args = {
			tag = {
				name = L["Tag"],
				desc = L["Tag to show for the current event."],
				type = 'text',
				usage = L["<Tag>"],
				get = getTag,
				set = setTag,
				passValue = category .. ";" .. name,
			},
			color = {
				name = L["Color"],
				desc = L["Color of the text for the current event."],
				type = 'color',
				get = getColor,
				set = setColor,
				passValue = category .. ";" .. name,
			},
			sticky = category == "Notification" and {
				name = L["Sticky"],
				desc = L["Whether the current event should be classified as \"Sticky\""],
				type = 'toggle',
				get = getSticky,
				set = setSticky,
				passValue = category .. ";" .. name,
			} or nil,
		}
	}
end
Parrot.RegisterCombatEvent = Parrot_CombatEvents.RegisterCombatEvent

local handler__translation
local handler__info
local function handler(literal)
	local inner = literal:sub(2, -2)
	local value = handler__translation[inner]
	if value then
		if type(value) == "function" then
			return tostring(value(handler__info))
		else
			return tostring(handler__info[value])
		end
	else
		return "[" .. inner:gsub("(%b[])", handler) .. "]"
	end
end

local absorbTranslation = { Amount = function(info)
	return "|cffffff00" .. info.absorbAmount .. "|r"
end }
local blockTranslation = { Amount = function(info)
	return "|cff7f00ff" .. info.blockAmount .. "|r"
end }
local resistTranslation = { Amount = function(info)
	return "|cff7f007f" .. info.resistAmount .. "|r"
end }
local vulnerableTranslation = { Amount = function(info)
	return "|cff7f7fff" .. info.vulnerableAmount .. "|r"
end }
local overhealTranslation = { Amount = function(info)
	return "|cff00af7f" .. info.overhealAmount .. "|r"
end }

function Parrot_CombatEvents:TriggerCombatEvent(category, name, info)
	self = Parrot_CombatEvents -- so people can do Parrot:TriggerCombatEvent
	if not Parrot:IsModuleActive(self) then
		return
	end
	local data = combatEvents[category]
	if not data then
		return
	end
	data = data[name]
	if not name then
		return
	end
	local sticky = false
	if data.canCrit then
		sticky = info.isCrit
	elseif category == "Notification" then
		sticky = self.db.profile[category][name].sticky
		if sticky == nil then
			sticky = data.sticky
		end
	end
	local text = self.db.profile[category][name].tag or data.defaultTag
	handler__translation = data.tagTranslations
	handler__info = info
	if handler__translation then
		text = text:gsub("(%b[])", handler)
	end
	local t = new(text)
	local absorbAmount = info.absorbAmount
	if absorbAmount and absorbAmount >= 1 then
		handler__translation = absorbTranslation
		t[#t+1] = L[" ([Amount] absorbed)"]:gsub("(%b[])", handler)
	end
	local blockAmount = info.blockAmount
	if blockAmount and blockAmount >= 1 then
		handler__translation = blockTranslation
		t[#t+1] = L[" ([Amount] blocked)"]:gsub("(%b[])", handler)
	end
	local resistAmount = info.resistAmount
	if resistAmount and resistAmount >= 1 then
		handler__translation = resistTranslation
		t[#t+1] = L[" ([Amount] resisted)"]:gsub("(%b[])", handler)
	end
	local vulnerableAmount = info.vulnerableAmount
	if vulnerableAmount and vulnerableAmount >= 1 then
		handler__translation = vulnerableTranslation
		t[#t+1] = L[" ([Amount] vulnerable)"]:gsub("(%b[])", handler)
	end
	local overhealAmount = info.overhealAmount
	if overhealAmount and overhealAmount >= 1 then
		handler__translation = overhealTranslation
		t[#t+1] = L[" ([Amount] overheal)"]:gsub("(%b[])", handler)
	end
	if info.isGlancing then
		t[#t+1] = L[" (|cff%02x%02x%02xglancing|r)"]:format(1*255, 0*255, 0*255)
	end
	if info.isCrushing then
		t[#t+1] = L[" (|cff%02x%02x%02xcrushing|r)"]:format(0.5*255, 0*255, 0*255)
	end	
	handler__translation = nil
	handler__info = nil
	Parrot_Display:ShowMessage(table.concat(t), category, sticky, self.db.profile[category][name].color or data.color)
	t = del(t)
end
Parrot.TriggerCombatEvent = Parrot_CombatEvents.TriggerCombatEvent

local damageTypeToColor = {
	["Physical"] = "ffffff",
	["Holy"] = "ffff7f",
	["Fire"] = "ff7f7f",
	["Nature"] = "7fff7f",
	["Frost"] = "7f7fff",
	["Shadow"] = "7f007f",
	["Arcane"] = "ff7fff",
}

local coloredDamageAmount = function(info) return ("|cff%s%s|r"):format(damageTypeToColor[info.damageType], info.amount) end

------------------------------------------------------------------------------
-- Incoming events -----------------------------------------------------------
------------------------------------------------------------------------------
Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Damage",
	localName = L["Melee Damage"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
	},
	color = "ff0000", -- red
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Misses",
	localName = L["Melee Misses"],
	defaultTag = L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Dodges",
	localName = L["Melee Dodges"],
	defaultTag = L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Parries",
	localName = L["Melee Parries"],
	defaultTag = L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Blocks",
	localName = L["Melee Blocks"],
	defaultTag = L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Absorbs",
	localName = L["Melee Absorbs"],
	defaultTag = L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Immunes",
	localName = L["Melee Immunes"],
	defaultTag = L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill Damage",
	localName = L["Skill Damage"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName_not = false,
		isDoT = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	},
	color = "ff0000", -- red
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill DoTs",
	localName = L["Skill DoTs"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName_not = false,
		isDoT = true,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	},
	color = "ff0000", -- red
	throttle = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability Misses",
	localName = L["Ability Misses"],
	defaultTag = "([Skill]) " .. L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability Dodges",
	localName = L["Ability Dodges"],
	defaultTag = "([Skill]) " .. L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability Parries",
	localName = L["Ability Parries"],
	defaultTag = "([Skill]) " .. L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability Blocks",
	localName = L["Ability Blocks"],
	defaultTag = "([Skill]) " .. L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
}


Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Spell Resists",
	localName = L["Spell Resists"],
	defaultTag = "([Skill]) " .. L["Resist!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "7f007f", -- purple
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill Absorbs",
	localName = L["Skill Absorbs"],
	defaultTag = "([Skill]) " .. L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill Immunes",
	localName = L["Skill Immunes"],
	defaultTag = "([Skill]) " .. L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill Reflects",
	localName = L["Skill Reflects"],
	defaultTag = "([Skill]) " .. L["Reflect!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "7f007f", -- purple
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Spell Interrupts",
	localName = L["Skill Interrupts"],
	defaultTag = "([Skill]) " .. L["Interrupt!"],
	parserEvent = {
		eventType = "Interrupt",
		recipientID = "player",
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Heals",
	localName = L["Heals"],
	defaultTag = "([Skill] - [Name]) +[Amount]",
	parserEvent = {
		eventType = "Heal",
		recipientID = "player",
		isHoT = false,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
		Amount = "realAmount",
	},
	color = "00ff00", -- green
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Heals over Time",
	localName = L["Heals over Time"],
	defaultTag = "([Skill] - [Name]) +[Amount]",
	parserEvent = {
		eventType = "Heal",
		recipientID = "player",
		isHoT = true,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
		Amount = "realAmount",
	},
	throttle = true,
	color = "00ff00", -- green
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Environmental Damage",
	localName = L["Environmental Damage"],
	defaultTag = "-[Amount] [Type]",
	parserEvent = {
		eventType = "Environmental",
		recipientID = "player",
	},
	tagTranslations = {
		Amount = coloredDamageAmount,
		Type = "hazardTypeLocal",
	},
	throttle = true,
	color = "ff0000", -- red
}

------------------------------------------------------------------------------
-- Outgoing events -----------------------------------------------------------
------------------------------------------------------------------------------
Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Damage",
	localName = L["Melee Damage"],
	defaultTag = "[Amount]",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
	},
	color = "ffffff", -- white
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Misses",
	localName = L["Melee Misses"],
	defaultTag = L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Dodges",
	localName = L["Melee Dodges"],
	defaultTag = L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Parries",
	localName = L["Melee Parries"],
	defaultTag = L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Blocks",
	localName = L["Melee Blocks"],
	defaultTag = L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Absorbs",
	localName = L["Melee Absorbs"],
	defaultTag = L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Immunes",
	localName = L["Melee Immunes"],
	defaultTag = L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Evades",
	localName = L["Melee Evades"],
	defaultTag = L["Evade!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill Damage",
	localName = L["Skill Damage"],
	defaultTag = "[Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
		isDoT = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill DoTs",
	localName = L["Skill DoTs"],
	defaultTag = "[Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
		isDoT = true,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	},
	throttle = true,
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability Misses",
	localName = L["Ability Misses"],
	defaultTag = L["Miss!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability Dodges",
	localName = L["Ability Dodges"],
	defaultTag = L["Dodge!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability Parries",
	localName = L["Ability Parries"],
	defaultTag = L["Parry!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ffffff", -- white
}


Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability Blocks",
	localName = L["Ability Blocks"],
	defaultTag = L["Block!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Spell Resists",
	localName = L["Spell Resists"],
	defaultTag = L["Resist!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "7f7fb2", -- blue-gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill Absorbs",
	localName = L["Skill Absorbs"],
	defaultTag = L["Absorb!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill Immunes",
	localName = L["Skill Immunes"],
	defaultTag = L["Immune!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill Reflects",
	localName = L["Skill Reflects"],
	defaultTag = L["Reflect!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Spell Interrupts",
	localName = L["Skill Interrupts"],
	defaultTag = L["Interrupt!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Interrupt",
		sourceID = "player",
		recipientID_not = "player",
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
}


Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill Evades",
	localName = L["Skill Evades"],
	defaultTag = L["Evade!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Heals",
	localName = L["Heals"],
	defaultTag = "+[Amount] ([Skill] - [Name])",
	parserEvent = {
		eventType = "Heal",
		sourceID = "player",
		recipientID_not = "player",
		isHoT = false,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
		Amount = "realAmount",
	},
	color = "00ff00", -- green
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Heals over Time",
	localName = L["Heals over Time"],
	defaultTag = "+[Amount] ([Skill] - [Name])",
	parserEvent = {
		eventType = "Heal",
		sourceID = "player",
		recipientID_not = "player",
		isHoT = true,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
		Amount = "realAmount",
	},
	throttle = true,
	color = "00ff00", -- green
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Melee Damage",
	localName = L["Pet Melee Damage"],
	defaultTag = "Pet [Amount]",
	parserEvent = {
		eventType = "Damage",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
	},
	color = "ff7f00", -- orange
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Melee Misses",
	localName = L["Pet Melee Misses"],
	defaultTag = L["Pet Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Melee Dodges",
	localName = L["Pet Melee Dodges"],
	defaultTag = L["Pet Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Melee Parries",
	localName = L["Pet Melee Parries"],
	defaultTag = L["Pet Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Melee Blocks",
	localName = L["Pet Melee Blocks"],
	defaultTag = L["Pet Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Melee Absorbs",
	localName = L["Pet Melee Absorbs"],
	defaultTag = L["Pet Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "7f7fff", -- light blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Melee Immunes",
	localName = L["Pet Melee Immunes"],
	defaultTag = L["Pet Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "7f7fff", -- light blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Melee Evades",
	localName = L["Pet Melee Evades"],
	defaultTag = L["Pet Evade!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ff7fff", -- pink
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Skill Damage",
	localName = L["Pet Skill Damage"],
	defaultTag = "Pet [Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Ability Misses",
	localName = L["Pet Ability Misses"],
	defaultTag = L["Pet Miss!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Ability Dodges",
	localName = L["Pet Ability Dodges"],
	defaultTag = L["Pet Dodge!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Ability Parries",
	localName = L["Pet Ability Parries"],
	defaultTag = L["Pet Parry!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Ability Blocks",
	localName = L["Pet Ability Blocks"],
	defaultTag = L["Pet Block!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Spell Resists",
	localName = L["Pet Spell Resists"],
	defaultTag = L["Pet Resist!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "7f7fb2", -- blue-gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Skill Absorbs",
	localName = L["Pet Skill Absorbs"],
	defaultTag = L["Pet Absorb!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "7f7fff", -- light blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Skill Immunes",
	localName = L["Pet Skill Immunes"],
	defaultTag = L["Pet Immune!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "7f7fff", -- light blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Skill Reflects",
	localName = L["Pet Skill Reflects"],
	defaultTag = L["Pet Reflect!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "7f7fff", -- light blue
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Skill Evades",
	localName = L["Pet Skill Evades"],
	defaultTag = L["Evade!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ff7fff", -- pink
}

------------------------------------------------------------------------------
-- Notification events -------------------------------------------------------
------------------------------------------------------------------------------

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Debuff Gains",
	localName = L["Debuff Gains"],
	defaultTag = "([Name])",
	blizzardEvent = "SpecialEvents_PlayerDebuffGained",
	tagTranslations = {
		Name = 1,
	},
	color = "007f7f", -- dark cyan
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Buff Gains",
	localName = L["Buff Gains"],
	defaultTag = "([Name])",
	blizzardEvent = "SpecialEvents_PlayerBuffGained",
	tagTranslations = {
		Name = 1,
	},
	color = "b2b200", -- dark yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Item Buff Gains",
	localName = L["Item Buff Gains"],
	defaultTag = "([Name])",
	tagTranslations = {
		Name = 1,
	},
	color = "b2b2b2", -- gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Debuff Fades",
	localName = L["Debuff Fades"],
	defaultTag = "-([Name])",
	blizzardEvent = "SpecialEvents_PlayerDebuffLost",
	tagTranslations = {
		Name = 1,
	},
	color = "00d8d8", -- cyan
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Buff Fades",
	localName = L["Buff Fades"],
	defaultTag = "-([Name])",
	blizzardEvent = "SpecialEvents_PlayerBuffLost",
	tagTranslations = {
		Name = 1,
	},
	color = "e5e500", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Item Buff Fades",
	localName = L["Item Buff Fades"],
	defaultTag = "-([Name])",
	tagTranslations = {
		Name = 1,
	},
	color = "e5e5e5", -- light gray
}

local function GetItemBuffName(slot)
	Gratuity:SetInventoryItem("player", slot)
	
	local _, _, buffName = Gratuity:Find("^([^%(]+) %(%d+ [^%)]+%)$", nil, nil, false, true)
	return buffName
end

local mainhandItemBuff
local offhandItemBuff
local function refreshItemBuffs()
	local mainhand, _, _, offhand = GetWeaponEnchantInfo()
	mainhandItemBuff = mainhand and GetItemBuffName(GetInventorySlotInfo("MainHandSlot"))
	offhandItemBuff = offhand and GetItemBuffName(GetInventorySlotInfo("SecondaryHandSlot"))
end
onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_CombatEvents:ScheduleRepeatingEvent("Parrot_CombatEvents:ItemBuffCheck", Parrot_CombatEvents.ItemBuffCheck, 0.2, Parrot_CombatEvents)
	
	refreshItemBuffs()
end
function Parrot_CombatEvents:ItemBuffCheck()
	local oldMainhandItemBuff = mainhandItemBuff
	local oldOffhandItemBuff = offhandItemBuff
	refreshItemBuffs()
	if oldMainhandItemBuff ~= mainhandItemBuff then
		if oldMainhandItemBuff then
			-- loss
			local info = new(oldMainhandItemBuff)
			self:TriggerCombatEvent("Notification", "Item Buff Fades", info)
			info = del(info)
		end
		if mainhandItemBuff then
			-- gain
			local info = new(mainhandItemBuff)
			self:TriggerCombatEvent("Notification", "Item Buffs", info)
			info = del(info)
		end
	end
	if oldOffhandItemBuff ~= offhandItemBuff then
		if oldOffhandItemBuff then
			-- loss
			local info = new(oldOffhandItemBuff)
			self:TriggerCombatEvent("Notification", "Item Buff Fades", info)
			info = del(info)
		end
		if offhandItemBuff then
			-- gain
			local info = new(offhandItemBuff)
			self:TriggerCombatEvent("Notification", "Item Buffs", info)
			info = del(info)
		end
	end
end

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Enter combat",
	localName = L["Enter combat"],
	defaultTag = "+Combat",
	blizzardEvent = "PLAYER_REGEN_DISABLED",
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Leave combat",
	localName = L["Leave combat"],
	defaultTag = "-Combat",
	blizzardEvent = "PLAYER_REGEN_ENABLED",
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Power Gain",
	localName = L["Power Gain"],
	defaultTag = L["+[Amount] [Type]"],
	parserEvent = {
		eventType = "Gain",
		attribute_in = { "Mana", "Focus", "Rage", "Energy" },
		recipientID = "player",
	},
	tagTranslations = {
		Amount = function(info)
			return info.amountGained or info.amount
		end,
		Type = function(info)
			return info.attributeGainedLocal or info.attributeLocal
		end,
		Skill = "abilityName",
		Name = function(info)
			return info.sourceGainedName or info.sourceName
		end,
	},
	color = "ffff00", -- yellow
}

onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_CombatEvents:RegisterParserEvent({
		eventType = "Leech",
		sourceGainedID = "player",
		attributeGained_in = { "Mana", "Focus", "Rage", "Energy" },
	}, function()
		Parrot_CombatEvents:TriggerEvent("Notification", "Power Gain", false, info)
	end)
end

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Power Loss",
	localName = L["Power Loss"],
	defaultTag = L["-[Amount] [Type]"],
	parserEvent = {
		eventType_in = { "Drain", "Leech" },
		attribute_in = { "Mana", "Focus", "Rage", "Energy" },
		recipientID = "player",
	},
	tagTranslations = {
		Amount = "amount",
		Type = "attributeLocal",
		Skill = "abilityName",
		Name = "sourceName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Combo Point Gain",
	localName = L["Combo Point Gain"],
	defaultTag = L["[Num] CP"],
	tagTranslations = {
		Num = 1
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Combo Points Full",
	localName = L["Combo Points Full"],
	defaultTag = L["[Num] CP Finish It!"],
	tagTranslations = {
		Num = 1
	},
	color = "ff7f00", -- orange
}

onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_CombatEvents:RegisterEvent("PLAYER_COMBO_POINTS", function()
		local num = GetComboPoints()
		if num == 0 then
			return
		end
		local info = new(num)
		Parrot_CombatEvents:TriggerCombatEvent("Notification", num == 5 and "Combo Points Full" or "Combo Point Gain", info)
		info = del(info)
	end)
end

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Honor Gains",
	localName = L["Honor Gains"],
	defaultTag = "+[Amount] " .. HONOR_CONTRIBUTION_POINTS,
	parserEvent = {
		eventType = "Honor",
		amount_gt = 0,
		isHonorable = true,
	},
	tagTranslations = {
		Amount = "amount",
		Name = "sourceName",
		Rank = "sourceRank",
	},
	color = "7f7fb2", -- blue-gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Reputation Gains",
	localName = L["Reputation Gains"],
	defaultTag = "+[Amount] " .. REPUTATION .. " ([Faction])",
	parserEvent = {
		eventType = "Reputation",
		amount_gt = 0,
	},
	tagTranslations = {
		Amount = "amount",
		Faction = "faction",
	},
	color = "7f7fb2", -- blue-gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Reputation Losses",
	localName = L["Reputation Losses"],
	defaultTag = "-[Amount] " .. REPUTATION .. " ([Faction])",
	parserEvent = {
		eventType = "Reputation",
		amount_lt = 0,
	},
	tagTranslations = {
		Amount = function(info) return -info.amount end,
		Faction = "faction",
	},
	color = "7f7fb2", -- blue-gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Skill Gains",
	localName = L["Skill Gains"],
	defaultTag = "[Skill]: [Amount]",
	parserEvent = {
		eventType = "Skill",
	},
	tagTranslations = {
		Skill = "abilityName",
		Amount = "amount",
	},
	color = "5555ff", -- semi-light blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Experience Gains",
	localName = L["Experience Gains"],
	defaultTag = "[Amount] " .. XP,
	parserEvent = {
		eventType = "Experience",
		recipientID = "player",
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = "amount",
	},
	color = "bf4ccc", -- magenta
	sticky = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Player Killing Blows",
	localName = L["Player Killing Blows"],
	defaultTag = L["Killing Blow!"] .. " ([Name])",
	parserEvent = {
		eventType = "Death",
		sourceID = "player",
		recipientID_not = "player",
		recipientPvP = true,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = function(info) return info.abilityName or PLAYERSTAT_MELEE_COMBAT end,
	},
	color = "5555ff", -- semi-light blue
	sticky = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "NPC Killing Blows",
	localName = L["NPC Killing Blows"],
	defaultTag = L["Killing Blow!"] .. " ([Name])",
	parserEvent = {
		eventType = "Death",
		sourceID = "player",
		recipientID_not = "player",
		recipientPvP = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = function(info) return info.abilityName or PLAYERSTAT_MELEE_COMBAT end,
	},
	color = "5555ff", -- semi-light blue
	sticky = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Soul Shard Gains",
	localName = L["Soul Shard Gains"],
	defaultTag = "+[Name]",
	parserEvent = {
		eventType = "Create",
		itemString_match = "item:6265:[%-%d:]+"
	},
	tagTranslations = {
		Name = "itemName",
	},
	color = "990099", -- purple
	sticky = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Extra Attacks",
	localName = L["Extra Attacks"],
	defaultTag = L["%s!"]:format("[Skill]"),
	parserEvent = {
		eventType = "Extra Attack",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
	sticky = true,
}
