local VERSION = tonumber(("$Revision: 43175 $"):match("%d+"))

local Parrot = Parrot
local Parrot_Display = Parrot:NewModule("Display", "AceEvent-2.0")
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-08 02:18:54 -0400 (Sun, 08 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local L = Parrot:L("Parrot_Display")

local SharedMedia -- AceLibrary("SharedMedia-1.0")

local new, del = Parrot.new, Parrot.del

local ParrotFrame

local Parrot_AnimationStyles
function Parrot_Display:OnEnable()
	Parrot_AnimationStyles = Parrot:GetModule("AnimationStyles")
	SharedMedia = AceLibrary("SharedMedia-1.0")
	if not ParrotFrame then
		ParrotFrame = CreateFrame("Frame", "ParrotFrame", UIParent)
		ParrotFrame:SetFrameStrata("HIGH")
		ParrotFrame:SetToplevel(true)
		ParrotFrame:SetPoint("CENTER")
		ParrotFrame:SetWidth(0.0001)
		ParrotFrame:SetHeight(0.0001)
	end
end

function Parrot_Display:OnDisable()
end

local scrollAreas = {
	["Notification"] = {
		align = "CENTER",
		animationStyle = "Straight",
		stickyAnimationStyle = "Pow",
		reverse = false,
		height = 150,
		xOffset = 0,
		yOffset = 100,
	},
	["Incoming"] = {
		align = "RIGHT",
		animationStyle = "Left parabola",
		stickyAnimationStyle = "Pow",
		reverse = true,
		height = 260,
		xOffset = -60,
		yOffset = -160,
	},
	["Outgoing"] = {
		align = "LEFT",
		animationStyle = "Right parabola",
		stickyAnimationStyle = "Pow",
		reverse = true,
		height = 260,
		xOffset = 60,
		yOffset = -160,
	},
}

local freeFontStrings = {}
local wildFontStrings
local fontString_num = 0
local fontStringIDs = {}

function Parrot_Display:ShowMessage(text, scrollArea, sticky, color)
	self = Parrot_Display -- for those who do Parrot:ShowMessage
	if not Parrot:IsModuleActive(self) then
		return
	end
	if scrollArea == nil and sticky == nil then
		sticky = true
	end
	if not scrollAreas[scrollArea] then
		if scrollAreas["Notification"] then
			scrollArea = "Notification"
		else
			scrollArea = next(scrollAreas)
			if not scrollArea then
				return
			end
		end
	end
	local fs = next(freeFontStrings)
	if fs then
		fs:SetAlpha(1)
		fs:ClearAllPoints()
		freeFontStrings[fs] = nil
	else
		fontString_num = fontString_num + 1
		fs = ParrotFrame:CreateFontString("ParrotFrameFontString" .. fontString_num, "ARTWORK", "MasterFont")
	end
	
	fs:SetFont(SharedMedia:Fetch('font', Parrot.db.profile.font), Parrot.db.profile.fontSize)
	if color then
		local num = tonumber(color, 16)
		fs:SetTextColor(math.floor(num/256^2)/255, (math.floor(num/256)%256)/255, (num%256)/255)
	else
		fs:SetTextColor(1, 1, 1)
	end
	fs:SetText(text)
	fs.start = GetTime()
	scrollArea = scrollAreas[scrollArea]
	fs.scrollArea = scrollArea
	local animationStyle
	if sticky then
		animationStyle = scrollArea.stickyAnimationStyle
	else
		animationStyle = scrollArea.animationStyle
	end
	local func, cleanup, overlap = Parrot_AnimationStyles:GetAnimationStyle(animationStyle)
	fs:SetPoint("BOTTOM", 0, 0)
	if not wildFontStrings then
		wildFontStrings = new()
		self:ScheduleRepeatingEvent("Parrot_OnUpdate", self.OnUpdate, 0, self)
	end
	local wildFontStrings_scrollArea = wildFontStrings[scrollArea]
	if not wildFontStrings_scrollArea then
		wildFontStrings_scrollArea = new()
		wildFontStrings[scrollArea] = wildFontStrings_scrollArea
	end
	local wildFontStrings_scrollArea_func = wildFontStrings_scrollArea[func]
	if not wildFontStrings_scrollArea_func then
		wildFontStrings_scrollArea_func = new()
		wildFontStrings_scrollArea[func] = wildFontStrings_scrollArea_func
		wildFontStrings_scrollArea_func.cleanup = cleanup
		wildFontStrings_scrollArea_func.length = 3
		wildFontStrings_scrollArea_func.overlap = overlap
	end
	local fontStringIDs_scrollArea = fontStringIDs[scrollArea]
	if not fontStringIDs_scrollArea then
		fontStringIDs_scrollArea = new()
		fontStringIDs[scrollArea] = fontStringIDs_scrollArea
	end
	local fontStringIDs_scrollArea_func = fontStringIDs_scrollArea[func]
	if not fontStringIDs_scrollArea_func then
		fontStringIDs_scrollArea_func = 1
	else
		fontStringIDs_scrollArea_func = fontStringIDs_scrollArea_func + 1
	end
	fontStringIDs_scrollArea[func] = fontStringIDs_scrollArea_func
	fs.id = fontStringIDs_scrollArea_func
	
	table.insert(wildFontStrings_scrollArea_func, 1, fs)
	fs:Show()
	self:OnUpdate(scrollArea, func)
end
Parrot.ShowMessage = Parrot_Display.ShowMessage

local function isOverlapping(alpha, bravo)
	if alpha:GetLeft() <= bravo:GetRight() and bravo:GetLeft() <= alpha:GetRight() and alpha:GetBottom() <= bravo:GetTop() and bravo:GetBottom() <= alpha:GetTop() then
		return true
	end
end

function Parrot_Display:OnUpdate()
	local now = GetTime()
	for scrollArea, u in pairs(wildFontStrings) do
		for animationStyle, t in pairs(u) do
			local t_len = #t
			local lastFS = nil
			for i, fs in ipairs(t) do
				local start, length = fs.start, t.length
				if start + length <= now then
					for j = i, t_len do
						local fs = t[j]
						local cleanup = t.cleanup
						if cleanup then
							cleanup(fs, scrollArea)
						end
						fs:Hide()
						t[j] = nil
						freeFontStrings[fs] = true
					end
					break
				end
				local percent = (now - start) / length
				if percent >= 0.75 then
					fs:SetAlpha((1-percent) * 4)
				end
				
				fs:ClearAllPoints()
				animationStyle(fs, scrollArea, percent, i, t_len, fs.id)
				
				if t.overlap and lastFS and isOverlapping(lastFS, fs) then
					local done = false
					local minimum = percent
					local maximum = 1
					local current
					while maximum - minimum > 0.01 do
						current = (maximum + minimum) / 2
						animationStyle(fs, scrollArea, current, i, t_len, fs.id)
						if isOverlapping(lastFS, fs) then
							minimum = current
						else
							maximum = current
						end
					end
					current = current + 0.01
					fs.start = -current * length + now
					animationStyle(fs, scrollArea, current, i, t_len, fs.id)
					for j = i+1, t_len do
						local v = t[j]
						if v.start > fs.start then
							v.start = fs.start
						end
					end
				end
				lastFS = fs
			end
			if not t[1] then
				u[animationStyle] = del(t)
			end
		end
		if not next(u) then
			wildFontStrings[scrollArea] = del(u)
		end
	end
	if not next(wildFontStrings) then
		wildFontStrings = del(wildFontStrings)
		self:CancelScheduledEvent("Parrot_OnUpdate")
	end
end
